package engine;
import javax.swing.JPanel;

import model.ChisqTest;
import model.Table;

/**
 * Interface define methods for data analysis
 * 
 * @author Pavel Vojtas
 * @version 1.0
 */
public interface Engine {
	
	/**
	 * Enumeration, which represented types of plot
	 * @author Pavel Vojtas
	 *
	 */
	public enum PLOT_TYPES {ABS, REL, REL100};
	
	/**
	 * Enumeration, which represented types of columns
	 * @author Pavel Vojtas
	 *
	 */
	public enum VAR_TYPES {NUM, FACTOR};
	
	/**
	 * Enumeration, which represented histogram types (normal, logarithm)
	 * @author Pavel Vojtas
	 *
	 */
	public enum HIST_TYPES {NORM, LOG}
	
	/**
	 * Enumeration, which represented type of column 
	 * separation in data file. 
	 * 
	 * @author Pavel Vojtas
	 *
	 */
	public enum FILE_TYPE {COMMA, SEMICOLON}
	
	/**
	 * Return instance of JPanel with plot.
	 * 
	 * @return plotGraphPanel JPanel
	 */
	public JPanel getPlotGraphPanel();
	
	/**
	 * Set instance of JPanel with plot.
	 * 
	 * @param plotGraphPanel JPanel
	 */
	public void setPlotGraphPanel(JPanel plotGraphPanel);	
	
	/**
	 * Create histogram for specific column, but not plot it.
	 * 
	 * @param colName String - names of column
	 * @param breaks Integer - number of breaks
	 * @param type {@link HIST_TYPES} - histogram type
	 */
	public void histogram(String colName, Integer breaks, HIST_TYPES type);
	
	/**
	 * Draw box plot for specific columns with specific title.
	 * 
	 * @param title String - title of plot
	 * @param colNames String - names of columns
	 */
	public void createBoxPlot(String title, String ... colNames);
	
	/**
	 * Draw bar plot for specific columns with specific title and labels.
	 * 
	 * @param type {@link PLOT_TYPES} - represent type of plot
	 * @param title String - title of plot
	 * @param xLabel String - label of x axe
	 * @param yLabel String - label of y axe
	 * @param colNames String - names of columns
	 */
	public void createBarPlot(PLOT_TYPES type, String title, String xLabel, String yLabel, String ... colNames);
	
	/**
	 * Draw pie plot for specific column with specific title.
	 * 
	 * @param type {@link PLOT_TYPES} - represent type of plot
	 * @param title String - title of plot
	 * @param colName String - name of specific column
	 */
	public void createPiePlot(PLOT_TYPES type, String title, String colName);
	
	/**
	 * Draw histogram for specific numeric column.
	 * 
	 * @param type {@link PLOT_TYPES} - types of plots
	 * @param title String - title of plot
	 * @param xLabel String - label of x axe
	 * @param yLabel String - label of y axe
	 * @param colName String - names of column
	 * @return boolean TRUE if histogram is plotted, FALSE otherwise.
	 */
	public boolean createHistogram(PLOT_TYPES type, String title, String xLabel, String yLabel, String colName);
	
	/**
	 * Return name of categories after categorize.
	 * 
	 * @param colName - name of column
	 * @return values String[] - names of categories
	 */
	public String[] getCategories(String colName);
	
	/**
	 * Return name of categories before categorized.
	 * 
	 * @param colName String - names of column
	 * @return values String[] - names of categories
	 */
	public String[] getDefaultCategories(String colName);
	
	/**
	 * Create and return table from imported data
	 * 
	 * @return table {@link Table} - data table
	 */
	public Table getTable();
	
	/**
	 * Create and return table with specific columns from imported data
	 * 
	 * @param columnNames String[] - specific columns
	 * @return table {@link Table} - data table
	 */
	public Table getTable(String[] columnNames);
	
	/**
	 * Create and return contingency table for variable Z(X, Y)
	 * 
	 * @param colName1 String - name of X variable
	 * @param colName2 String - name of Y variable
	 * @return table {@link Table} - contingency table
	 */
	public Table getContingencyTable(String colName1, String colName2);
	
	/**
	 * Create and return frequency table for categoric variable. 
	 * Table contains absolute frequency, relative frequency, 
	 * relative frequency in % and cumulative frequency
	 * 
	 * @param col String - name of specific column
	 * @return table {@link Table} - frequency table
	 */
	public Table getFrequencyTable(String col);
	
	/**
	 * Create and return quantile table for numeric variables. 
	 * Table contains 0%, 25%, 50%, 75% and 100% quantiles
	 * for each variable
	 * 
	 * @param cols String - name of specific columns
	 * @return table {@link Table} - quantile table
	 */
	public Table getQuantileTable(String ... cols);

	/**
	 * Create and return table with description statistic values.
	 * The values are Modal category, Mean, Variation ratio, 
	 * Nominal variance, Entropy, Variation range, IRQ, Variation, 
	 * Standard deviation
	 * 
	 * @param cols String - name of specific columns
	 * @return table {@link Table} - characteristic table
	 */
	public Table getCharacteristicTable(String ...  cols);
	
	/**
	 * Change names of categories for specific category column
	 * 
	 * @param values String - names of categories
	 * @param colName String - name of column
	 */
	public void changeCategoriesNames(String[] values, String colName);
	
	/**
	 * Import table of data
	 * 
	 * @param filePath String - location of imported file
	 * @param fileType {@link FILE_TYPE} - type of file
	 * @return TRUE if load is success, FALSE otherwise
	 */
	public boolean loadTable(String filePath, FILE_TYPE fileType);

	/**
	 * Export plots to specific path with specific size
	 * 
	 * @param filePath String - location of export file
	 * @param width int - width of exported plot
	 * @param height int  - height of exported plot
	 */
	public void savePlots(String filePath, int width, int height);

	/**
	 * Set specific colors to plots
	 * 
	 * @param colors String[] - names of colors
	 */
	public void setColors(String[] colors);

	/**
	 * Set missing values to specific column
	 * 
	 * @param value String - category to set as missing
	 * @param colName String - name of column
	 */
	public void setNAValue(String value, String colName);
	
	/**
	 * Convert type of column to different type of column
	 * 
	 * @param colName String - name of column
	 * @param convert {@link VAR_TYPES} - type to convert
	 * @return boolean TRUE if convert is success, FALSE otherwise. 
	 */
	public boolean convertVariable(String colName, VAR_TYPES convert);
	
	/**
	 * Compute and create report of Pearson's Chi-squared Test,
	 * for testing independency
	 * 
	 * @return chisqTest {@link ChisqTest}
	 */
	public ChisqTest chisqDependencyTest();

}