package gui;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.JTableHeader;

import engine.Engine;
import engine.Engine.FILE_TYPE;
import gui.components.Button;
import gui.components.Label;
import gui.dialog.ChooseNADialog;
import gui.dialog.ChooseVariableDialog;
import gui.model.MyTableModel;
import utils.FileUtils;
import utils.Language;

/**
 * Frame with source data table and functions for 
 * choose missing values and choose detail columns.
 * 
 * This frame is specific variant from base class {@link MainFrame}.
 * 
 * @author Pavel Vojtas
 * @version 1.0
 */
public class DataFrame extends MainFrame {

	private static final long serialVersionUID = 1L;
	
	private Button btnDetail;
	private Button btnLoad;
	private JPanel centerPanel;
	private File dataFile;
	private Label lblFilePath;
	private JTable table;
	private MyTableModel tableModel;
	private JTextField tfPath;
	private JPanel topPanel;
	private String[] chooseVariables = new String[1];
	
	/**
	 * Create frame with specific language and engine for show data file.
	 * 
	 * @param engine {@link Engine} - engine for data analyze
	 * @param lang {@link Language} - language for load labels
	 * @throws IOException - Exception for invalid language, or invalid language file
	 */
	public DataFrame(Engine engine, Language lang) throws IOException {
		super(engine, lang);
				
		createTopPanel();
		pack();
		
		initListeners();
		//setMinimumSize(new Dimension(getWidth()+25, getHeight()));
	}

	/**
	 * Show exit dialog. If YES option is choose, the application 
	 * will exit itself with all children frames.
	 */
	protected void closeWindow(){
		int result = JOptionPane.showOptionDialog(rootPane, langProp.getProperty("DIALOG_TEXT_CLOSE"), "", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, new String[]{langProp.getProperty("DIALOG_OPTION_YES"), langProp.getProperty("DIALOG_OPTION_NO")}, null);
		if(result == JOptionPane.YES_OPTION){
			System.exit(0);
		}
	}

	/**
	 * Create center panel with JTable 
	 * with data from imported file.
	 */
	private void createCenterPanel(){
		centerPanel = new JPanel(new BorderLayout());
		
		tableModel = new MyTableModel(engine.getTable());
		table = new JTable(tableModel);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(500, 500));
		centerPanel.add(scrollPane);
		add(centerPanel, BorderLayout.CENTER);
		
		table.setColumnSelectionAllowed(false);
		table.setRowSelectionAllowed(false);
		table.getTableHeader().setReorderingAllowed(false);
		table.getTableHeader().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				table.setColumnSelectionAllowed(true);
				table.setRowSelectionAllowed(false);
				JTableHeader header = (JTableHeader) e.getSource();
				int column = header.columnAtPoint(e.getPoint());
				table.setColumnSelectionInterval(column, column);
				
				if(e.getClickCount() == 2 && e.getButton() == 1){
					int selected = table.getSelectedColumn();
					
					String[] level = engine.getDefaultCategories(tableModel.getColumnName(selected));
					String value = null;
					try {
						value = ChooseNADialog.showDialog(level, langProp.getProperty("DIALOG_NA_TITLE"));
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					
					if(value == null){
						table.setColumnSelectionAllowed(false);
						table.setRowSelectionAllowed(false);
						return;
					}
					else {
						engine.setNAValue(value, tableModel.getNames()[selected]);
						tableModel = new MyTableModel(engine.getTable());
						table.setModel(tableModel);
						pack();
						table.setColumnSelectionAllowed(false);
						table.setRowSelectionAllowed(false);
					}
				}
				table.setColumnSelectionAllowed(false);
				table.setRowSelectionAllowed(false);
			}
		});
		btnDetail.setEnabled(true);
	}
	
	/**
	 * Create panel for import file
	 */
	private void createTopPanel(){
		topPanel = new JPanel();
		topPanel.setLayout(new FlowLayout());
		
		lblFilePath = new Label("LABEL_FILEPATH", language);
		tfPath = new JTextField(40);
		btnLoad = new Button("BUTTON_LOAD", language);
		btnDetail = new Button("BUTTON_DETAIL", language);
		btnDetail.setEnabled(false);
		
		topPanel.add(lblFilePath);
		addObject(lblFilePath);
		topPanel.add(tfPath);
		topPanel.add(btnLoad);
		addObject(btnLoad);
		topPanel.add(btnDetail);
		addObject(btnDetail);
		
		add(topPanel, BorderLayout.NORTH);
	}
	
	/**
	 * Initialize additional listeners for 
	 * new gui objects.
	 */
	private void initListeners(){
		btnLoad.addActionListener(new ActionListener() {
		
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Object[] openFile = FileUtils.showCSVOpenDialog();
				if(openFile != null){
					dataFile = (File) openFile[0];
					FILE_TYPE type = (FILE_TYPE) openFile[1];
					if(engine.loadTable(dataFile.getAbsolutePath(), type)){
						tfPath.setText(dataFile.getAbsolutePath());
						if(centerPanel != null) remove(centerPanel);
						createCenterPanel();
						pack();
						setMinimumSize(new Dimension(getWidth(), getHeight()));
					}
				}
			}
		});
	
		btnLoad.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(btnLoad.getNotes());
			}
		});
		
		btnDetail.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				List<String> result = null;
				try {
					result = ChooseVariableDialog.showDialog(tableModel.getNames(), engine, langProp.getProperty("DIALOG_CHOOSE_VARIABLE_TITLE"));
				} catch (IOException e) {
					e.printStackTrace();
					return;
				}
							
				try {
					if(result != null) new DetailFrame(engine, language, result.toArray(chooseVariables));
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		
		btnDetail.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(btnDetail.getNotes());
			}
		});
	}

}