package gui;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.ButtonGroup;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

import engine.Engine;
import utils.Language;
import utils.LanguageUtils;
import gui.components.GuiObject;
import gui.components.Menu;
import gui.components.RadioButtonMenuItem;
import gui.components.MenuItem;
import gui.panels.StatusBar;

/**
 * Abstract frame represents basic window. Blank window
 * with menu bar and menus for close window and change
 * language.
 * 
 * @author Pavel Vojtas
 * @version 1.0
 */
public abstract class MainFrame extends javax.swing.JFrame {

	private static final long serialVersionUID = 1L;
	
	protected Engine engine;
	private List<GuiObject> guiObjects = new ArrayList<GuiObject>();
	protected Properties langProp = new Properties();
	protected Language language;
	protected JMenuBar menuBar;
	protected Menu menuConfig;
	protected Menu menuFile;
	private RadioButtonMenuItem menuItemCzech;
	private RadioButtonMenuItem menuItemEnglish;
	private MenuItem menuItemExit;
	private Menu menuLang;
	protected StatusBar statusBar;

	/**
	 * Create frame with specific language and engine for data analyze.
	 * 
	 * @param engine {@link Engine} - engine for data analyze
	 * @param lang {@link Language} - language for load labels
	 * @throws IOException - Exception for invalid language, or invalid language file 
	 */
	public MainFrame(Engine engine, Language lang) throws IOException{
		this.engine = engine;
		this.language = lang;
		
		getDefaultLang();
		
		initMainGui();
		initMainListeners();
		
		setTitle(langProp.getProperty("APP_TITLE"));
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}

	/**
	 * Add object which implements interface {@link GuiObject} 
	 * and store it in collection guiObjects ArrayList.
	 * 
	 * @param object {@link GuiObject} - Graphic object
	 */
	public void addObject(GuiObject object){
		guiObjects.add(object);
	}
	
	/**
	 * Abstract method for close actual frame
	 */
	protected abstract void closeWindow();
	
	/**
	 * Create basic menus
	 */
	private void createMainMenu(){
		menuBar = new JMenuBar();
		
		menuFile = new Menu("MENU_FILE", language);
		addObject(menuFile);
		menuItemExit = new MenuItem("MENU_FILE_CLOSE", language);
		addObject(menuItemExit);
		menuConfig = new Menu("MENU_CONFIG", language);
		addObject(menuConfig);
		menuLang = new Menu("MENU_CONFIG_LANG", language);
		addObject(menuLang);
		menuItemCzech = new RadioButtonMenuItem("MENU_CONFIG_LANG_CS", language);
		addObject(menuItemCzech);
		menuItemEnglish = new RadioButtonMenuItem("MENU_CONFIG_LANG_EN", language);
		addObject(menuItemEnglish);
		
		ButtonGroup group = new ButtonGroup();
		group.add(menuItemCzech);
		group.add(menuItemEnglish);
		
		menuFile.add(menuItemExit);
		menuConfig.add(menuLang);
		menuLang.add(menuItemCzech);
		menuLang.add(menuItemEnglish);
		menuBar.add(menuFile);
		menuBar.add(menuConfig);
		
		setJMenuBar(menuBar);
		
		if(language == Language.CZECH){
			menuItemCzech.setSelected(true);
		}
		else if(language == Language.ENGLISH){
			menuItemEnglish.setSelected(true);
		}
	}
	
	/**
	 * Create status bar {@link StatusBar} 
	 * with specific size
	 */
	private void createStatusBar(){
		statusBar = new StatusBar(20);
		add(statusBar, BorderLayout.SOUTH);
	}
	
	/**
	 * Get objects from collection guiObject ArrayList 
	 * and replace all text from specific language file and config file.
	 */
	private void fireLabels(){
		setTitle(langProp.getProperty("APP_TITLE"));
		for (int i = 0; i < guiObjects.size(); i++) {
			guiObjects.get(i).fireLabels(language);
		}
	}
	
	/**
	 * Load language from config file
	 * 
	 * @throws IOException - Exception in file i/o operations
	 */
	private void getDefaultLang() throws IOException{
		langProp.load(LanguageUtils.openLangFile(language));
	}
	
	/**
	 * Return actual language
	 * 
	 * @return language {@link Language} 
	 */
	public Language getLanguage(){
		return language;
	}
	
	/**
	 * Create basic gui objects
	 */
	private void initMainGui(){
		setLayout(new BorderLayout());
		
		createMainMenu();
		createStatusBar();

		pack();
	}

	/**
	 * Create basic listeners
	 */
	private void initMainListeners(){
		menuItemExit.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				closeWindow();
			}
		});
		
		addWindowListener(new WindowAdapter() {
			
			@Override
			public void windowClosing(WindowEvent arg0) {
				closeWindow();
			}
			
		});
		
		menuItemCzech.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				language = Language.CZECH;	
				try {
					getDefaultLang();
					setDefaultLang();
					fireLabels();
				} catch (IOException e) {
					JOptionPane.showMessageDialog(rootPane, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					language = Language.ENGLISH;
					e.printStackTrace();
				}
			}
		});
		
		menuItemEnglish.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				language = Language.ENGLISH;	
				try {
					getDefaultLang();
					setDefaultLang();
					fireLabels();
				} catch (IOException e) {
					JOptionPane.showMessageDialog(rootPane, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					language = Language.CZECH;
					e.printStackTrace();
				}
			}
		});
		
		menuFile.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuFile.getNotes());
			}
		});
		
		menuConfig.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuConfig.getNotes());
			}
		});
		
		menuItemExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuItemExit.getNotes());
			}
		});
		
		menuLang.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuLang.getNotes());
			}
		});
		
		menuItemCzech.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuItemCzech.getNotes());
			}
		});
		
		menuItemEnglish.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				statusBar.setStatusText(menuItemEnglish.getNotes());
			}
		});
		
		addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent e) {
				statusBar.setStatusText("");
			}
		});
	}
	
	/**
	 * Write actual language to config file
	 * 
	 * @throws IOException - Exception in file i/o operations
	 */
	private void setDefaultLang() throws IOException{
		if(language == Language.CZECH){
			LanguageUtils.writeConfigFile("language", "cs");
		}
		else if(language == Language.ENGLISH){
			LanguageUtils.writeConfigFile("language", "en");
		}
	}

}