package gui.dialog;

import gui.components.CheckBox;
import gui.components.RadioButton;
import gui.components.TitleBorder;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import utils.LanguageUtils;

/**
 * Modal dialog for analysis options.
 * 
 * @author Pavel Vojtas
 * @version 1.0
 *
 */
public class AnalyzeOptionsDialog extends JDialog{
	

	private static final long serialVersionUID = 1L;
	public static final String VARIABLE = "var";
	public static final String GROUP1_HIST = "hist";
	public static final String GROUP1_HIST_USER = "histUser";
	public static final String GROUP1_HIST_DEFAULT = "histDefault";
	public static final String GROUP1_FREQUENCY_TABLE = "freq";
	public static final String GROUP2_KVANTILE_TABLE = "kvant";
	public static final String GROUP2_CHARACTER_TABLE = "char";
	public static final String HIST_BREAKS = "breaks";
	public static final int SELECT = 0;
	public static final int NO_SECECT = 1;
	
	private JSlider slider;
	private JButton button;
	private int count = 1000;
	private JLabel actual;
	private JTextField tfActual;
	private String[] variables;
	private JList<String> list;
	
	private Map<String, Integer> values;
	private RadioButton btnHistDef;
	private RadioButton btnHistUser;
	private CheckBox btnHist;
	private CheckBox btnFreq;
	private JPanel group1Panel;
	private CheckBox btnKvant;
	private CheckBox btnChar;
	private JPanel group2Panel;
	private JPanel sliderPanel;
	
	/**
	 * Create dialog for specific columns and defined language
	 * 
	 * @param variables String[] - columns names for analysis
	 * @param langProp Properties - language properties
	 * @throws IOException - exception from language file
	 */
	public AnalyzeOptionsDialog(String[] variables, Properties langProp) throws IOException {
		super();
		setModal(true);
		setTitle(langProp.getProperty("DIALOG_ANALYZE_TITLE"));
		
		this.variables = variables;
		createGUI();
		initListeners();
	}

	/**
	 * Initialize listeners
	 */
	private void initListeners() {
		button.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				values = new HashMap<String, Integer>();
				values.put(VARIABLE, new Integer(list.getSelectedIndex())); 
				values.put(GROUP1_HIST, btnHist.isSelected() ? SELECT : NO_SECECT);
				values.put(GROUP1_HIST_DEFAULT, btnHistDef.isSelected() ? SELECT : NO_SECECT);
				values.put(GROUP1_HIST_USER, btnHistUser.isSelected() ? SELECT : NO_SECECT);
				values.put(GROUP1_FREQUENCY_TABLE, btnFreq.isSelected() ? SELECT : NO_SECECT);
				values.put(GROUP2_KVANTILE_TABLE, btnKvant.isSelected() ? SELECT : NO_SECECT);
				values.put(GROUP2_CHARACTER_TABLE, btnChar.isSelected() ? SELECT : NO_SECECT);
				try {
					Integer value = Integer.parseInt(tfActual.getText());
					if(value.intValue() == 0) throw new NumberFormatException();
					
					values.put(HIST_BREAKS, value-1);

				} catch (NumberFormatException e) {
					return;
				}
				dispose();
			}
		});
		
		slider.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent arg0) {
				tfActual.setText(""+slider.getValue());
			}
		});
		
		btnHist.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				btnHistDef.setEnabled(btnHist.isSelected());
				btnHistUser.setEnabled(btnHist.isSelected());
				if(btnHistUser.isSelected()){
					tfActual.setEnabled(btnHist.isSelected());
					slider.setEnabled(btnHist.isSelected());
				}
			}
		});
		
		btnHistUser.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				tfActual.setEnabled(true);
				slider.setEnabled(true);
			}
		});
		
		btnHistDef.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				tfActual.setEnabled(false);
				slider.setEnabled(false);
			}
		});
	}
	
	/**
	 * Create basic gui structure
	 * @throws IOException - language file exception
	 */
	private void createGUI() throws IOException {
		setPreferredSize(new Dimension(400,330));
		setLayout(new BorderLayout());
		setResizable(false);
		
		createListPanel();
		createOptionsPanel();
		
		button = new JButton("OK");
		
		JPanel btnPanel = new JPanel();
		btnPanel.setPreferredSize(new Dimension(60, 30));
		btnPanel.add(button);
		add(btnPanel, BorderLayout.SOUTH);
		
		btnHistDef.setEnabled(false);
		btnHistUser.setEnabled(false);
		slider.setEnabled(false);
		tfActual.setEnabled(false);
		
		pack();
		
	}
	
	/**
	 * Create basic option panel.
	 * 
	 * @throws IOException - language file exception
	 */
	private void createOptionsPanel() throws IOException {
		JPanel optPanel = new JPanel(new BorderLayout());
		BoxLayout optLayout = new BoxLayout(optPanel, BoxLayout.Y_AXIS);
		optPanel.setLayout(optLayout);
		
		group1Panel = new JPanel();
		BoxLayout group1Layout = new BoxLayout(group1Panel, BoxLayout.Y_AXIS);
		group1Panel.setLayout(group1Layout);
		
		btnHistDef = new RadioButton("DIALOG_ANALYZE_HIST_DEFAULT", LanguageUtils.getLanguage());
		btnHistDef.setSelected(true);
		btnHistUser = new RadioButton("DIALOG_ANALYZE_HIST_USER", LanguageUtils.getLanguage());
		
		ButtonGroup histGroup = new ButtonGroup();
		histGroup.add(btnHistDef);
		histGroup.add(btnHistUser);
		
		btnHist = new CheckBox("DIALOG_ANALYZE_HIST", LanguageUtils.getLanguage());
		btnFreq = new CheckBox("DIALOG_ANALYZE_FREQ", LanguageUtils.getLanguage());
		
		group1Panel.add(btnFreq);
		group1Panel.add(btnHist);
		group1Panel.add(btnHistDef);
		group1Panel.add(btnHistUser);
		sliderPanel = createSliderPanel();
		group1Panel.add(sliderPanel);
		group1Panel.setBorder(new TitleBorder("DIALOG_ANALYZE_GROUP1_TITLE", LanguageUtils.getLanguage()));
		
		group2Panel = new JPanel();
		BoxLayout group2Layout = new BoxLayout(group2Panel, BoxLayout.Y_AXIS);
		group2Panel.setLayout(group2Layout);
		
		btnKvant = new CheckBox("DIALOG_ANALYZE_KVANT", LanguageUtils.getLanguage());
		btnChar = new CheckBox("DIALOG_ANALYZE_CHAR", LanguageUtils.getLanguage());
		
		group2Panel.add(btnKvant);
		group2Panel.add(btnChar);
		group2Panel.setBorder(new TitleBorder("DIALOG_ANALYZE_GROUP2_TITLE", LanguageUtils.getLanguage()));
		
		optPanel.add(group1Panel);
		optPanel.add(group2Panel);
		add(optPanel);
	}
	
	/**
	 * Create panel with default JSlider.
	 * 
	 * @return panel JPanel - panel with slider
	 */
	private JPanel createSliderPanel() {
		JPanel panelFull = new JPanel(new BorderLayout());
		JPanel panel = new JPanel(new FlowLayout());
		actual = new JLabel("Actual: ");
		tfActual = new JTextField(20);
		panel.add(actual);
		panel.add(tfActual);
		
		slider = new JSlider(JSlider.HORIZONTAL, 2, count, (count-1)/2);
		slider.setMajorTickSpacing((count-1)/10);
		slider.setPaintTicks(true);
		tfActual.setText(""+slider.getValue());
		
		Hashtable<Integer, JLabel> labelTable = new Hashtable<>();
		labelTable.put( new Integer( 2 ), new JLabel("2") );
		labelTable.put( new Integer( count ), new JLabel(""+count) );
		slider.setLabelTable( labelTable );
		slider.setPaintLabels(true);
		
		panelFull.add(panel, BorderLayout.NORTH);
		panelFull.add(slider, BorderLayout.CENTER);
		return panelFull;
	}
	
	/**
	 * Create list panel with defined columns names.
	 * 
	 * @throws IOException - language file exception
	 */
	private void createListPanel() throws IOException {
		JPanel listPanel = new JPanel(new BorderLayout());
		listPanel.setBorder(new TitleBorder("DIALOG_ANALYZE_LIST_TITLE", LanguageUtils.getLanguage()));
		listPanel.setPreferredSize(new Dimension(100, 300));
		list = new JList<>(variables);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane scrollPane = new JScrollPane(list);
		listPanel.add(scrollPane, BorderLayout.CENTER);
		
		add(listPanel, BorderLayout.WEST);
	}
	
	/**
	 * Return structure with specific values from analysis setting, 
	 * under specific String key.
	 * 
	 * @return values Map - setting of analysis
	 */
	public Map<String, Integer> getValue() {
		return values;
	}
	
	/**
	 * Show and create option dialog. It return structure with analysis settings values.
	 * 
	 * @param variable String[] - columns names for analysis
	 * @param langProp Properties - language properties
	 * @return values Map - setting of analysis
	 */
	public static Map<String, Integer> showDialog(String[] variable, Properties langProp){
		AnalyzeOptionsDialog dialog;
		try {
			dialog = new AnalyzeOptionsDialog(variable, langProp);
			dialog.setVisible(true);
			return dialog.getValue();
		} catch (IOException e) {
			return null;
		}	
	}
}
