package gui.dialog;

import engine.Engine;
import engine.Engine.VAR_TYPES;
import gui.components.Button;
import gui.components.RadioButton;
import gui.components.TitleBorder;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;

import utils.LanguageUtils;

/**
 * Model dialog for choose column for detail view and analyzing. 
 * Also provide column type transformation.
 * 
 * @author Pavel Vojtas
 * @version 1.0
 *
 */
public class ChooseVariableDialog extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private String[] variables;
	private JList<String> sourceList;
	private List<VAR_TYPES> typeList;
	private Engine engine;
	private JList<Object> destList;
	private List<String> varList;
	private Button okButton;
	private Button addButton;
	private RadioButton btnNumeric;
	private boolean approveClose = false;

	private RadioButton btnFactor;

	private Button delButton;

	/**
	 * Create dialog.
	 * 
	 * @param variables String[] - all columns from dataset
	 * @param engine {@link Engine} - engine for data analyze
	 * @param title String - dialog title
	 * @throws IOException - language file exceptions
	 */
	public ChooseVariableDialog(String[] variables, Engine engine, String title) throws IOException {
		super();
		setModal(true);
		setTitle(title);
		
		this.variables = variables;
		this.engine = engine;
		this.typeList = new ArrayList<VAR_TYPES>();
		this.varList = new ArrayList<String>();
		
		createGUI();
		initListeners();
	}
	
	/**
	 * Initialize listeners.
	 */
	private void initListeners() {
		okButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(!varList.isEmpty()) approveClose = true;
				dispose();
			}
		});
		
		addButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				VAR_TYPES type = null;
				if(btnNumeric.isSelected()){
					type = VAR_TYPES.NUM;
				}
				else if(btnFactor.isSelected()){
					type = VAR_TYPES.FACTOR;
				}
				
				boolean result = engine.convertVariable(sourceList.getSelectedValue(), type);
				
				if(!result){
					JOptionPane.showMessageDialog(rootPane, "Convert error", "error", JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				if(!varList.contains(sourceList.getSelectedValue())){
					typeList.add(type);
					varList.add(sourceList.getSelectedValue());
					destList.setListData(varList.toArray());
				}	
			}
		});
		
		delButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int index = destList.getSelectedIndex();
				if(index <= varList.size() && index >= 0){
					varList.remove(index);
					typeList.remove(index);
					destList.setListData(varList.toArray());
				}	
			}
		});
		
	}
	
	/**
	 * Create gui structure.
	 * 
	 * @throws IOException - language file exceptions
	 */
	private void createGUI() throws IOException {
		//setPreferredSize(new Dimension(560, 200));
		setLayout(new BorderLayout());
		setResizable(false);
		
		createSourceListPanel();
		createDestinationListPanel();
		createChooseTypePanel();
		pack();
	}
	
	/**
	 * Create panel for column type transformation.
	 * 
	 * @throws IOException - language file exceptions
	 */
	private void createChooseTypePanel() throws IOException {
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridBagLayout());
		centerPanel.setBorder(new TitleBorder("DIALOG_CHOOSE_VARIABLE_CHOOSEAREA", LanguageUtils.getLanguage()));
		
		btnNumeric = new RadioButton("DIALOG_CHOOSE_VARIABLE_BTN_NUMERIC", LanguageUtils.getLanguage());
		btnFactor = new RadioButton("DIALOG_CHOOSE_VARIABLE_BTN_FACTOR", LanguageUtils.getLanguage());
		btnFactor.setSelected(true);
		
		ButtonGroup btnGroup = new ButtonGroup();
		btnGroup.add(btnNumeric);
		btnGroup.add(btnFactor);
		
		centerPanel.add(btnNumeric, c); c.gridy++;
		centerPanel.add(btnFactor, c); c.gridy++; c.fill = GridBagConstraints.HORIZONTAL; c.insets = new Insets(10, 10, 10, 10);
		
		okButton = new Button("DIALOG_CHOOSE_VARIABLE_BTN_OK", LanguageUtils.getLanguage());
		addButton = new Button("DIALOG_CHOOSE_VARIABLE_BTN_ADD", LanguageUtils.getLanguage());
		delButton = new Button("DIALOG_CHOOSE_VARIABLE_BTN_DEL", LanguageUtils.getLanguage());
		centerPanel.add(addButton, c); c.gridy++; c.insets = new Insets(0, 10, 4, 10); 
		centerPanel.add(delButton, c); c.gridy++;  c.insets = new Insets(10, 10, 10, 10);
		centerPanel.add(new JSeparator(), c); c.gridy++;
		centerPanel.add(okButton, c);
		
		add(centerPanel, BorderLayout.CENTER);
	}
	/**
	 * Create panel with list of choose columns.
	 * 
	 * @throws IOException - language file exceptions
	 */
	private void createDestinationListPanel() throws IOException {
		JPanel rightPanel = new JPanel(new BorderLayout());
		rightPanel.setPreferredSize(new Dimension(150, 200));
		rightPanel.setBorder(new TitleBorder("DIALOG_CHOOSE_VARIABLE_DESTLIST", LanguageUtils.getLanguage()));
		
		destList = new JList<>();
		JScrollPane scrollPane = new JScrollPane(destList);
		rightPanel.add(scrollPane);
		
		add(rightPanel, BorderLayout.EAST);
	}
	
	/**
	 * Create panel with list of all columns
	 * 
	 * @throws IOException - language file exceptions
	 */
	private void createSourceListPanel() throws IOException {
		JPanel leftPanel = new JPanel(new BorderLayout());
		leftPanel.setPreferredSize(new Dimension(150, 200));
		leftPanel.setBorder(new TitleBorder("DIALOG_CHOOSE_VARIABLE_SOURCELIST", LanguageUtils.getLanguage()));
		
		sourceList = new JList<>(variables);
		JScrollPane scrollPane = new JScrollPane(sourceList);
		leftPanel.add(scrollPane);
		
		add(leftPanel, BorderLayout.WEST);
	}
	
	/**
	 * Return types of choose columns.
	 * 
	 * @return types List - choose columns type
	 */
	public List<VAR_TYPES> getTypeList() {
		return typeList;
	}
	
	/**
	 * Return choose columns.
	 * 
	 * @return types List - choose columns list
	 */
	public List<String> getVarList() {
		return varList;
	}
	
	/**
	 * Return true if dialog is close with approve option, false otherwise.
	 * 
	 * @return approveClose boolean
	 */
	public boolean isApproveClose() {
		return approveClose;
	}
	
	/**
	 * Create and show dialog. It return choose columns list
	 * 
	 * @param variables String[] - all columns from dataset
	 * @param engine {@link Engine} - engine for data analyze
	 * @param title String - dialog title
	 * @return types List - choose columns
	 * @throws IOException - language file exceptions
	 */
	public static List<String> showDialog(String[] variables, Engine engine, String title) throws IOException{
		ChooseVariableDialog dialog = new ChooseVariableDialog(variables, engine, title);
		dialog.setVisible(true);
		
		if(!dialog.isApproveClose()){
			JOptionPane.showMessageDialog(dialog, "Musite vybrat promennou", "Warning", JOptionPane.WARNING_MESSAGE);
			return null;
		}
		
		return dialog.getVarList();
	}
}
