package gui.panels;

import gui.PlotFrame;
import gui.components.Button;
import gui.components.GuiObject;
import gui.components.Label;
import gui.components.TitleBorder;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import model.Boolean;
import utils.Language;
import engine.Engine;

/**
 * Abstract panel. Contain basic plot components 
 * such as table for color choose, textfields 
 * for plot title and draw button.
 * 
 * @author Pavel Vojtas
 * @version 1.0
 *
 */
public abstract class PlotPanel extends JPanel implements GuiObject{
	
	private static final long serialVersionUID = 1L;
	
	private List<GuiObject> guiObjects = new ArrayList<GuiObject>();
	
	protected JFrame parent;
	private JPanel plotPanel;
	private JPanel graphPanel;
	private Button btnDraw;
	private Label lblTitle;
	private TitleBorder plotBorder;
	private model.Boolean newWindow = new model.Boolean(false);
	
	protected Engine engine;
	protected JTextField tfTitle;
	protected String[] colName;

	protected Language language;
	
	/**
	 * Create universal plot panel.
	 * 
	 * @param engine {@link Engine} - engine for data analyze
	 * @param lang {@link Language} - language for load labels
	 * @param parent JFrame - parent frame
	 * @param colName String[] - columns names
	 */
	public PlotPanel(Engine engine, Language lang, JFrame parent, String[] colName) {
		this.engine = engine;
		this.parent = parent;
		this.colName = colName;
		this.language = lang;
		
		setLayout(new BorderLayout());
		
		createGUI();
		initListeners();
	}
	
	/**
	 * Initialize basic listeners for gui objects.
	 */
	private void initListeners() {
		btnDraw.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				plotWindow();
			}
		});
	}
	
	/**
	 * Create basic gui objects.
	 */
	private void createGUI() {	
		plotPanel = new JPanel();
		BoxLayout layout = new BoxLayout(plotPanel, BoxLayout.Y_AXIS);
		plotPanel.setLayout(layout);
		
		btnDraw = new Button("BUTTON_DRAW", language);
		addGuiObject(btnDraw);
		
		lblTitle = new Label("LABEL_PLOT_TITLE", language);
		addGuiObject(lblTitle);
		
		tfTitle = new JTextField(30);
		
		JPanel titlePanel = new JPanel(new FlowLayout());
		titlePanel.add(lblTitle);
		titlePanel.add(tfTitle);
		
		plotPanel.add(titlePanel);
		
		createComponents();
		
		plotPanel.add(btnDraw);
		add(plotPanel, BorderLayout.WEST);
		plotBorder = new TitleBorder("LABEL_PLOT", language);
		addGuiObject(plotBorder);
		setBorder(plotBorder);
	}
	
	@Override
	public Component add(Component arg0) {
		return plotPanel.add(arg0);
	}
	
	/**
	 * Add object which implements interface {@link GuiObject} 
	 * and store it in collection guiObjects ArrayList.
	 * 
	 * @param object {@link GuiObject} - Graphic object
	 */
	public void addGuiObject(GuiObject object){
		guiObjects.add(object);
	}
	
	/** 
	 * Abstract method for additional gui components.
	 */
	protected abstract void createComponents();
	
	/**
	 * Abstract method for actions to draw plot.
	 */
	protected abstract boolean drawPlot();
	
	/** 
	 * Create plot in new windows or same window.
	 */
	private void plotWindow(){
		boolean status = drawPlot();
		if(!status) return;
		graphPanel = engine.getPlotGraphPanel();
		if(newWindow.isValue()){
			PlotFrame plotFrame;
			try {
				plotFrame = new PlotFrame(engine, language);
				plotFrame.add(graphPanel);
				plotFrame.pack();
				plotFrame.setVisible(true);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else {
			add(graphPanel, BorderLayout.EAST);
			parent.pack();
		}
	}

	/**
	 * Return draw button.
	 * 
	 * @return button JButton - draw button
	 */
	public JButton getBtnDraw() {
		return btnDraw;
	}
	
	/**
	 * Change all labels for specified language
	 * 
	 * @param lang {@link Language} - specified language
	 */
	public void fireLabels(Language lang){
		for (int i = 0; i < guiObjects.size(); i++) {
			guiObjects.get(i).fireLabels(lang);;
		}
	}
	
	/**
	 * Return actual language settings
	 * 
	 * @return language {@link Language} - actual language
	 */
	public Language getLanguage(){
		return language;
	}
	
	/**
	 * Set language
	 * 
	 * @param language {@link Language}
	 */
	public void setLanguage(Language language){
		this.language = language;
	}
	
	/**
	 * not implemented (empty string)
	 */
	public String getName(){
		return "";
	}

	/**
	 * not implemented (empty string)
	 */
	public String getNotes(){
		return "";
	}

	/**
	 * Return true if plot is in new windows, false otherwise.
	 * 
	 * @return {@link Boolean}
	 */
	public model.Boolean getNewWindow() {
		return newWindow;
	}
	
	/**
	 * Set true to plot in new window, false otherwise.
	 * 
	 * @param newWindow {@link Boolean}
	 */
	public void setNewWindow(model.Boolean newWindow) {
		this.newWindow = newWindow;
	}	
}
